# Chapter  8 - section 8.1

# BRUGS program that generates BUGS data file 
# for running the GLM with Zellner prior configuration
# for the 4 competing models M1, M2, M3 and M4

# ------------------------------------------------------------------


rm(list=ls())

library(rbugs)
library(boot)


# Load data
# ------------------------------------------

data_GLMpSm1 <- read.table(file = "data_GLMpSm1_table.txt", header = T, sep = "")

# Select data for Year >= 1973 (Year = 1 = 1964 ; Year = 40 = 2003) 

data_GLMpSm1 <- subset(data_GLMpSm1, subset=c(Year>=10))
n = length(data_GLMpSm1$y1)

# Constants common to all models
# ------------------------------------------

# tau ~gamma(a,b)
a <- 1
b <- 1

# c : relative weight of the data vs prior
# c=1 : same weight prior / data
# large c ==> high importance given to the data
c <- n
# c <- 1/n



# Baseline model M0 (pSm1 ~ year + SeaAge + Region)
# ------------------------------------------

model.version <- "M0"
dim_theta <- 5

# -----------------------------------
theta0 <- rep(x=0, times = dim_theta)

X <- matrix(NA,n,dim_theta)
X[,1] <- rep(1,n)
X[,2] <- data_GLMpSm1$Year

R1 <- 1*(data_GLMpSm1$Region==1)
R2 <- 1*(data_GLMpSm1$Region==2)
R3 <- 1*(data_GLMpSm1$Region==3)
X[,3] <- R1-R3
X[,4] <- R2-R3

A1 <- 1*(data_GLMpSm1$SeaAge==1)
A2 <- 1*(data_GLMpSm1$SeaAge==2)
X[,5] <- A1-A2

P0 <- t(X)%*%X

data <- c(as.list(data_GLMpSm1[-c(1,2,3)]), list(n=n, theta0=theta0, a = a, b = b, c = c, P0 = P0, X = X, k = dim_theta))
file_to_save <- paste("data_GLMpSm1_Zellner_",model.version,".txt", sep = "")
genDataFile(data, file_to_save)      




# Model M1 (pSm1 ~ year + SeaAge) - No Region
# ------------------------------------------

model.version <- "M1"
dim_theta <- 3

# -----------------------------------
theta0 <- rep(x=0, times = dim_theta)

X <- matrix(NA,n,dim_theta)
X[,1] <- rep(1,n)
X[,2] <- data_GLMpSm1$Year

A1 <- 1*(data_GLMpSm1$SeaAge==1)
A2 <- 1*(data_GLMpSm1$SeaAge==2)
X[,3] <- A1-A2

P0 <- t(X)%*%X

data <- c(as.list(data_GLMpSm1[-c(1,2,3)]), list(n=n, theta0=theta0, a = a, b = b, c = c, P0 = P0, X = X, k = dim_theta))
file_to_save <- paste("data_GLMpSm1_Zellner_",model.version,".txt", sep = "")
genDataFile(data, file_to_save)





# Model M2 (pSm1 ~ year + Region) - No SeaAge
# ------------------------------------------

model.version <- "M2"
dim_theta <- 4

# -----------------------------------
theta0 <- rep(x=0, times = dim_theta)

X <- matrix(NA,n,dim_theta)
X[,1] <- rep(1,n)
X[,2] <- data_GLMpSm1$Year

R1 <- 1*(data_GLMpSm1$Region==1)
R2 <- 1*(data_GLMpSm1$Region==2)
R3 <- 1*(data_GLMpSm1$Region==3)
X[,3] <- R1-R3
X[,4] <- R2-R3

P0 <- t(X)%*%X

data <- c(as.list(data_GLMpSm1[-c(1,2,3)]), list(n=n, theta0=theta0, a = a, b = b, c = c, P0 = P0, X = X, k = dim_theta))
file_to_save <- paste("data_GLMpSm1_Zellner_",model.version,".txt", sep = "")
genDataFile(data, file_to_save)





# Model M3 (pSm1 ~ year*SeaAge + Region) 
# ------------------------------------------

model.version <- "M3"
dim_theta <- 5

# -----------------------------------
theta0 <- rep(x=0, times = dim_theta)

X <- matrix(NA,n,dim_theta)
X[,1] <- rep(1,n)

A1 <- 1*(data_GLMpSm1$SeaAge==1)
A2 <- 1*(data_GLMpSm1$SeaAge==2)

X[,2] <- data_GLMpSm1$Year*A1
X[,3] <- data_GLMpSm1$Year*A2


R1 <- 1*(data_GLMpSm1$Region==1)
R2 <- 1*(data_GLMpSm1$Region==2)
R3 <- 1*(data_GLMpSm1$Region==3)
X[,4] <- R1-R3
X[,5] <- R2-R3
P0 <- t(X)%*%X

data <- c(as.list(data_GLMpSm1[-c(1,2,3)]), list(n=n, theta0=theta0, a = a, b = b, c = c, P0 = P0, X = X, k = dim_theta))
file_to_save <- paste("data_GLMpSm1_Zellner_",model.version,".txt", sep = "")
genDataFile(data, file_to_save)





# Model M4 (pSm1 ~ year*SeaAge + non common origin+ Region) 
# ------------------------------------------

model.version <- "M4"
dim_theta <- 6

# -----------------------------------
theta0 <- rep(x=0, times = dim_theta)

X <- matrix(NA,n,dim_theta)
#X[,1] <- rep(1,n)

A1 <- 1*(data_GLMpSm1$SeaAge==1)
A2 <- 1*(data_GLMpSm1$SeaAge==2)
X[,1] <- A1
X[,2] <- A2
X[,3] <- data_GLMpSm1$Year*A1
X[,4] <- data_GLMpSm1$Year*A2


R1 <- 1*(data_GLMpSm1$Region==1)
R2 <- 1*(data_GLMpSm1$Region==2)
R3 <- 1*(data_GLMpSm1$Region==3)
X[,5] <- R1-R3
X[,6] <- R2-R3
P0 <- t(X)%*%X

data <- c(as.list(data_GLMpSm1[-c(1,2,3)]), list(n=n, theta0=theta0, a = a, b = b, c = c, P0 = P0, X = X, k = dim_theta))
file_to_save <- paste("data_GLMpSm1_Zellner_",model.version,".txt", sep = "")
genDataFile(data, file_to_save)
                                                           